@@ -1,5 +1,5 @@
 /* Copyright (C) 1991, 1992, 1993, 1996, 1997, 1998, 1999, 2001, 2002, 2003,
-   2005, 2007, 2009 Free Software Foundation, Inc.
+   2005, 2007, 2009, 2010 Free Software Foundation, Inc.
 
    This file is part of the GNU C Library.
 
