@@ -1,7 +1,7 @@
 /*
  * augeas.c: the core data structure for storing key/value pairs
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -50,7 +50,7 @@ static const char *const static_nodes[][2] = {
     { AUGEAS_META_TREE "/version/save/mode[2]", AUG_SAVE_NEWFILE_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[3]", AUG_SAVE_NOOP_TEXT },
     { AUGEAS_META_TREE "/version/save/mode[4]", AUG_SAVE_OVERWRITE_TEXT },
-    { AUGEAS_META_TREE "/version/defvar", NULL }
+    { AUGEAS_META_TREE "/version/defvar/expr", NULL }
 };
 
 static const char *const errcodes[] = {
@@ -484,7 +484,7 @@ static int find_one_node(struct pathx *p, struct tree **match) {
 }
 
 int aug_get(const struct augeas *aug, const char *path, const char **value) {
-    struct pathx *p;
+    struct pathx *p = NULL;
     struct tree *match;
     int r;
 
@@ -497,6 +497,10 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
         *value = NULL;
 
     r = pathx_find_one(p, &match);
+    ERR_BAIL(aug);
+    ERR_THROW(r > 1, aug, AUG_EMMATCH, "There are %d nodes matching %s",
+              r, path);
+
     if (r == 1 && value != NULL)
         *value = match->value;
     free_pathx(p);
@@ -504,6 +508,7 @@ int aug_get(const struct augeas *aug, const char *path, const char **value) {
     api_exit(aug);
     return r;
  error:
+    free_pathx(p);
     api_exit(aug);
     return -1;
 }
@@ -521,6 +526,20 @@ int aug_defvar(augeas *aug, const char *name, const char *expr) {
         ERR_BAIL(aug);
         result = pathx_symtab_define(&(aug->symtab), name, p);
     }
+    ERR_BAIL(aug);
+
+    /* Record the definition of the variable */
+    struct tree *tree = tree_path_cr(aug->origin, 2, "augeas", "variables");
+    ERR_NOMEM(tree == NULL, aug);
+    if (expr == NULL) {
+        tree = tree_child(tree, name);
+        if (tree != NULL)
+            tree_unlink(tree);
+    } else {
+        tree = tree_child_cr(tree, name);
+        ERR_NOMEM(tree == NULL, aug);
+        tree_set_value(tree, expr);
+    }
  error:
     free_pathx(p);
     api_exit(aug);
@@ -600,6 +619,61 @@ int aug_set(struct augeas *aug, const char *path, const char *value) {
     return -1;
 }
 
+int aug_setm(struct augeas *aug, const char *base,
+             const char *sub, const char *value) {
+    struct pathx *bx = NULL, *sx = NULL;
+    struct error *err = err_of_aug(aug);
+    struct tree *bt, *st;
+    int result, r;
+
+    api_entry(aug);
+
+    bx = parse_user_pathx(aug, true, base);
+    ERR_BAIL(aug);
+
+    if (sub != NULL && STREQ(sub, "."))
+        sub = NULL;
+
+    result = 0;
+    for (bt = pathx_first(bx); bt != NULL; bt = pathx_next(bx)) {
+        if (sub != NULL) {
+            /* Handle subnodes of BT */
+            pathx_parse(bt, err, sub, true, aug->symtab, &sx);
+            ERR_BAIL(aug);
+            if (pathx_first(sx) != NULL) {
+                /* Change existing subnodes matching SUB */
+                for (st = pathx_first(sx); st != NULL; st = pathx_next(sx)) {
+                    r = tree_set_value(st, value);
+                    ERR_NOMEM(r < 0, aug);
+                    result += 1;
+                }
+            } else {
+                /* Create a new subnode matching SUB */
+                r = pathx_expand_tree(sx, &st);
+                if (r == -1)
+                    goto error;
+                r = tree_set_value(st, value);
+                ERR_NOMEM(r < 0, aug);
+                result += 1;
+            }
+            free_pathx(sx);
+            sx = NULL;
+        } else {
+            /* Set nodes matching BT directly */
+            r = tree_set_value(bt, value);
+            ERR_NOMEM(r < 0, aug);
+            result += 1;
+        }
+    }
+
+ done:
+    api_exit(aug);
+    return result;
+ error:
+    result = -1;
+    goto done;
+}
+
 int tree_insert(struct pathx *p, const char *label, int before) {
     struct tree *new = NULL, *match;
 
@@ -780,8 +854,6 @@ int tree_replace(struct tree *origin, const char *path, struct tree *sub) {
     list_for_each(s, sub) {
         s->parent = parent;
     }
-    if (sub->dirty)
-        tree_mark_dirty(parent);
     free_pathx(p);
     return 0;
  error:
