@@ -1,7 +1,7 @@
 /*
  * augtool.c:
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -590,6 +590,39 @@ static const struct command_def cmd_set_def = {
     "will appear last\n amongst their siblings"
 };
 
+static void cmd_setm(struct command *cmd) {
+    const char *base = arg_value(cmd, "base");
+    const char *sub  = arg_value(cmd, "sub");
+    const char *val  = arg_value(cmd, "value");
+    int r;
+
+    r = aug_setm(aug, base, sub, val);
+    err_check(cmd);
+    if (r == -1)
+        printf ("Failed\n");
+}
+
+static const struct command_opt_def cmd_setm_opts[] = {
+    { .type = CMD_PATH, .name = "base", .optional = false,
+      .help = "the base node" },
+    { .type = CMD_PATH, .name = "sub", .optional = false,
+      .help = "the subtree relative to the base" },
+    { .type = CMD_STR, .name = "value", .optional = false,
+      .help = "the value for the nodes" },
+    CMD_OPT_DEF_LAST
+};
+
+static const struct command_def cmd_setm_def = {
+    .name = "setm",
+    .opts = cmd_setm_opts,
+    .handler = cmd_setm,
+    .synopsis = "set the value of multiple nodes",
+    .help = "Set multiple nodes in one operation.  Find or create a node"
+    " matching SUB\n by interpreting SUB as a  path expression relative"
+    " to each node matching\n BASE. If SUB is '.', the nodes matching "
+    "BASE will be modified."
+};
+
 static void cmd_defvar(struct command *cmd) {
     const char *name = arg_value(cmd, "name");
     const char *path = arg_value(cmd, "expr");
@@ -686,9 +719,12 @@ static const struct command_def cmd_clear_def = {
 static void cmd_get(struct command *cmd) {
     const char *path = arg_value(cmd, "path");
     const char *val;
+    int r;
 
+    r = aug_get(aug, path, &val);
+    ERR_CHECK(cmd);
     printf("%s", path);
-    if (aug_get(aug, path, &val) != 1) {
+    if (r == 0) {
         printf(" (o)\n");
     } else if (val == NULL) {
         printf(" (none)\n");
@@ -894,6 +930,7 @@ static const struct command_def const *commands[] = {
     &cmd_rm_def,
     &cmd_save_def,
     &cmd_set_def,
+    &cmd_setm_def,
     &cmd_help_def,
     &cmd_def_last
 };
@@ -1039,7 +1076,7 @@ static void print_version_info(void) {
         goto error;
 
     fprintf(stderr, "augtool %s <http://augeas.net/>\n", version);
-    fprintf(stderr, "Copyright (C) 2009 Red Hat, Inc.\n");
+    fprintf(stderr, "Copyright (C) 2009-2010 David Lutterkort\n");
     fprintf(stderr, "License LGPLv2+: GNU LGPL version 2.1 or later\n");
     fprintf(stderr, "                 <http://www.gnu.org/licenses/lgpl-2.1.html>\n");
     fprintf(stderr, "This is free software: you are free to change and redistribute it.\n");
@@ -1092,7 +1129,7 @@ static int main_loop(void) {
             printf("\n");
             return ret;
         }
-        if (line[0] == '#')
+        if (*line == '\0' || *line == '#')
             continue;
 
         code = run_command(line);
