@@ -1,7 +1,7 @@
 /*
  * parser.c: parse a configuration file according to a grammar
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -37,9 +37,6 @@
 static const char *const short_iteration =
     "Iterated lens matched less than it should";
 
-#define assert_error(state, format, args ...) \
-    assert_error_at(__FILE__, __LINE__, &(state->info), format, ## args)
-
 struct seq {
     struct seq *next;
     const char *name;
@@ -47,7 +44,7 @@ struct seq {
 };
 
 struct state {
-    struct info       info;
+    struct info      *info;
     const char       *text;
     struct seq       *seqs;
     char             *key;
@@ -251,7 +248,7 @@ static void get_expected_error(struct state *state, struct lens *l) {
  */
 
 static char *token(struct state *state) {
-    assert(REG_MATCHED(state));
+    ensure0(REG_MATCHED(state), state->info);
     return strndup(REG_POS(state), REG_SIZE(state));
 }
 
@@ -274,20 +271,20 @@ static void regexp_match_error(struct state *state, struct lens *lens,
 }
 
 static void no_match_error(struct state *state, struct lens *lens) {
-    assert(lens->tag == L_KEY || lens->tag == L_DEL
-           || lens->tag == L_STORE);
+    ensure(lens->tag == L_KEY || lens->tag == L_DEL
+           || lens->tag == L_STORE, state->info);
     char *pat = regexp_escape(lens->ctype);
-    const char *lname;
+    const char *lname = "(lname)";
     if (lens->tag == L_KEY)
         lname = "key";
     else if (lens->tag == L_DEL)
         lname = "del";
     else if (lens->tag == L_STORE)
         lname = "store";
-    else
-        assert(0);
     get_error(state, lens, "no match for %s /%s/", lname, pat);
     free(pat);
+ error:
+    return;
 }
 
 /* Modifies STATE->REGS and STATE->NREG. The caller must save these
@@ -332,7 +329,7 @@ static void free_seqs(struct seq *seqs) {
 }
 
 static struct seq *find_seq(const char *name, struct state *state) {
-    assert(name != NULL);
+    ensure0(name != NULL, state->info);
     struct seq *seq;
 
     for (seq=state->seqs;
@@ -350,14 +347,15 @@ static struct seq *find_seq(const char *name, struct state *state) {
 }
 
 static struct tree *get_seq(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_SEQ);
+    ensure0(lens->tag == L_SEQ, state->info);
     struct seq *seq = find_seq(lens->string->str, state);
+    int r;
+
+    r = asprintf((char **) &(state->key), "%d", seq->value);
+    ERR_NOMEM(r < 0, state->info);
 
-    if (asprintf((char **) &(state->key), "%d", seq->value) == -1) {
-        // FIXME: We are out of memory .. find a way to report that
-        abort();
-    }
     seq->value += 1;
+ error:
     return NULL;
 }
 
@@ -367,7 +365,7 @@ static struct skel *parse_seq(struct lens *lens, struct state *state) {
 }
 
 static struct tree *get_counter(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_COUNTER);
+    ensure0(lens->tag == L_COUNTER, state->info);
     struct seq *seq = find_seq(lens->string->str, state);
     seq->value = 1;
     return NULL;
@@ -379,7 +377,7 @@ static struct skel *parse_counter(struct lens *lens, struct state *state) {
 }
 
 static struct tree *get_del(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_DEL);
+    ensure0(lens->tag == L_DEL, state->info);
     if (! REG_MATCHED(state)) {
         char *pat = regexp_escape(lens->ctype);
         get_error(state, lens, "no match for del /%s/", pat);
@@ -389,7 +387,7 @@ static struct tree *get_del(struct lens *lens, struct state *state) {
 }
 
 static struct skel *parse_del(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_DEL);
+    ensure0(lens->tag == L_DEL, state->info);
     struct skel *skel = NULL;
 
     skel = make_skel(lens);
@@ -401,10 +399,11 @@ static struct skel *parse_del(struct lens *lens, struct state *state) {
 }
 
 static struct tree *get_store(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_STORE);
+    ensure0(lens->tag == L_STORE, state->info);
+    ensure0(state->value == NULL, state->info);
+
     struct tree *tree = NULL;
 
-    assert(state->value == NULL);
     if (state->value != NULL)
         get_error(state, lens, "More than one store in a subtree");
     else if (! REG_MATCHED(state))
@@ -416,12 +415,24 @@ static struct tree *get_store(struct lens *lens, struct state *state) {
 
 static struct skel *parse_store(struct lens *lens,
                                 ATTRIBUTE_UNUSED struct state *state) {
-    assert(lens->tag == L_STORE);
+    ensure0(lens->tag == L_STORE, state->info);
+    return make_skel(lens);
+}
+
+static struct tree *get_value(struct lens *lens, struct state *state) {
+    ensure0(lens->tag == L_VALUE, state->info);
+    state->value = strdup(lens->string->str);
+    return NULL;
+}
+
+static struct skel *parse_value(struct lens *lens,
+                                ATTRIBUTE_UNUSED struct state *state) {
+    ensure0(lens->tag == L_VALUE, state->info);
     return make_skel(lens);
 }
 
 static struct tree *get_key(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_KEY);
+    ensure0(lens->tag == L_KEY, state->info);
     if (! REG_MATCHED(state))
         no_match_error(state, lens);
     else
@@ -435,7 +446,7 @@ static struct skel *parse_key(struct lens *lens, struct state *state) {
 }
 
 static struct tree *get_label(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_LABEL);
+    ensure0(lens->tag == L_LABEL, state->info);
     state->key = strdup(lens->string->str);
     return NULL;
 }
@@ -446,7 +457,8 @@ static struct skel *parse_label(struct lens *lens, struct state *state) {
 }
 
 static struct tree *get_union(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_UNION);
+    ensure0(lens->tag == L_UNION, state->info);
+
     struct tree *tree = NULL;
     int applied = 0;
     uint old_nreg = state->nreg;
@@ -468,7 +480,8 @@ static struct tree *get_union(struct lens *lens, struct state *state) {
 
 static struct skel *parse_union(struct lens *lens, struct state *state,
                                 struct dict **dict) {
-    assert(lens->tag == L_UNION);
+    ensure0(lens->tag == L_UNION, state->info);
+
     struct skel *skel = NULL;
     int applied = 0;
     uint old_nreg = state->nreg;
@@ -491,7 +504,8 @@ static struct skel *parse_union(struct lens *lens, struct state *state,
 }
 
 static struct tree *get_concat(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_CONCAT);
+    ensure0(lens->tag == L_CONCAT, state->info);
+
     struct tree *tree = NULL;
     uint old_nreg = state->nreg;
 
@@ -517,7 +531,7 @@ static struct tree *get_concat(struct lens *lens, struct state *state) {
 
 static struct skel *parse_concat(struct lens *lens, struct state *state,
                                  struct dict **dict) {
-    assert(lens->tag == L_CONCAT);
+    ensure0(lens->tag == L_CONCAT, state->info);
     struct skel *skel = make_skel(lens);
     uint old_nreg = state->nreg;
 
@@ -543,7 +557,7 @@ static struct skel *parse_concat(struct lens *lens, struct state *state,
 }
 
 static struct tree *get_quant_star(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_STAR);
+    ensure0(lens->tag == L_STAR, state->info);
     struct lens *child = lens->child;
     struct tree *tree = NULL, *tail = NULL;
     struct re_registers *old_regs = state->regs;
@@ -575,7 +589,7 @@ static struct tree *get_quant_star(struct lens *lens, struct state *state) {
 
 static struct skel *parse_quant_star(struct lens *lens, struct state *state,
                                      struct dict **dict) {
-    assert(lens->tag == L_STAR);
+    ensure0(lens->tag == L_STAR, state->info);
     struct lens *child = lens->child;
     struct skel *skel = make_skel(lens), *tail = NULL;
     struct re_registers *old_regs = state->regs;
@@ -608,7 +622,7 @@ static struct skel *parse_quant_star(struct lens *lens, struct state *state,
 }
 
 static struct tree *get_quant_maybe(struct lens *lens, struct state *state) {
-    assert(lens->tag == L_MAYBE);
+    ensure0(lens->tag == L_MAYBE, state->info);
     struct tree *tree = NULL;
 
     /* Check that our child matched. For a construct like (r)?, the
@@ -625,7 +639,7 @@ static struct tree *get_quant_maybe(struct lens *lens, struct state *state) {
 
 static struct skel *parse_quant_maybe(struct lens *lens, struct state *state,
                                       struct dict **dict) {
-    assert(lens->tag == L_MAYBE);
+    ensure0(lens->tag == L_MAYBE, state->info);
 
     struct skel *skel = NULL;
 
@@ -689,7 +703,7 @@ static void print_frames(struct rec_state *state) {
 
 ATTRIBUTE_PURE
 static struct frame *top_frame(struct rec_state *state) {
-    assert(state->fsize > 0);
+    ensure0(state->fsize > 0, state->state->info);
     return state->frames + state->fused - 1;
 }
 
@@ -708,7 +722,7 @@ static struct frame *push_frame(struct rec_state *state, struct lens *lens) {
         if (expand < 8)
             expand = 8;
         r = REALLOC_N(state->frames, state->fsize + expand);
-        ERR_NOMEM(r < 0, &state->state->info);
+        ERR_NOMEM(r < 0, state->state->info);
         state->fsize += expand;
     }
 
@@ -723,7 +737,7 @@ static struct frame *push_frame(struct rec_state *state, struct lens *lens) {
 }
 
 static struct frame *pop_frame(struct rec_state *state) {
-    assert(state->fused > 0);
+    ensure0(state->fused > 0, state->state->info);
 
     state->fused -= 1;
     if (state->fused > 0)
@@ -820,11 +834,11 @@ static void get_combine(struct rec_state *rec_state,
             while (tail->next != NULL) tail = tail->next;
 
         if (top->key != NULL) {
-            assert(key == NULL);
+            ensure(key == NULL, rec_state->state->info);
             key = top->key;
         }
         if (top->value != NULL) {
-            assert(value == NULL);
+            ensure(value == NULL, rec_state->state->info);
             value = top->value;
         }
     }
@@ -832,6 +846,8 @@ static void get_combine(struct rec_state *rec_state,
     top->tree = tree;
     top->key = key;
     top->value = value;
+ error:
+    return;
 }
 
 static void parse_combine(struct rec_state *rec_state,
@@ -851,7 +867,7 @@ static void parse_combine(struct rec_state *rec_state,
             while (tail->next != NULL) tail = tail->next;
         dict_append(&dict, top->dict);
         if (top->key != NULL) {
-            assert(key == NULL);
+            ensure(key == NULL, rec_state->state->info);
             key = top->key;
         }
     }
@@ -859,6 +875,8 @@ static void parse_combine(struct rec_state *rec_state,
     top->skel = skel;
     top->dict = dict;
     top->key = key;
+ error:
+    return;
 }
 
 static void visit_exit(struct lens *lens,
@@ -884,7 +902,7 @@ static void visit_exit(struct lens *lens,
             tree = make_tree(top->key, top->value, NULL, top->tree);
             ERR_NOMEM(tree == NULL, lens->info);
             top = pop_frame(rec_state);
-            assert(lens == top->lens);
+            ensure(lens == top->lens, state->info);
             state->key = top->key;
             state->value = top->value;
             pop_frame(rec_state);
@@ -898,7 +916,7 @@ static void visit_exit(struct lens *lens,
             dict = make_dict(top->key, top->skel, top->dict);
             ERR_NOMEM(dict == NULL, lens->info);
             top = pop_frame(rec_state);
-            assert(lens == top->lens);
+            ensure(lens == top->lens, state->info);
             state->key = top->key;
             pop_frame(rec_state);
             top = push_frame(rec_state, lens);
@@ -906,7 +924,7 @@ static void visit_exit(struct lens *lens,
             top->dict = dict;
         }
     } else if (lens->tag == L_CONCAT) {
-        assert(rec_state->fused >= lens->nchildren);
+        ensure(rec_state->fused >= lens->nchildren, state->info);
         for (int i = 0; i < lens->nchildren; i++) {
             struct frame *fr = nth_frame(rec_state, i);
             BUG_ON(lens->children[i] != fr->lens,
@@ -929,6 +947,12 @@ static void visit_exit(struct lens *lens,
             get_combine(rec_state, lens, n);
         else
             parse_combine(rec_state, lens, n);
+    } else if (lens->tag == L_MAYBE) {
+        uint n = (top_frame(rec_state)->lens == lens->child) ? 1 : 0;
+        if (rec_state->mode == M_GET)
+            get_combine(rec_state, lens, n);
+        else
+            parse_combine(rec_state, lens, n);
     } else {
         top_frame(rec_state)->lens = lens;
     }
@@ -949,19 +973,23 @@ static void visit_error(struct lens *lens, void *data, size_t pos,
 
 static struct frame *rec_process(enum mode_t mode, struct lens *lens,
                                  struct state *state) {
-    assert(lens->tag == L_REC && lens->jmt != NULL);
-
     uint end = REG_END(state);
     uint start = REG_START(state);
-    size_t len;
+    size_t len = 0;
     struct re_registers *old_regs = state->regs;
     uint old_nreg = state->nreg;
+    int r;
     struct jmt_visitor visitor;
     struct rec_state rec_state;
 
     MEMZERO(&rec_state, 1);
     MEMZERO(&visitor, 1);
 
+    if (lens->jmt == NULL) {
+        lens->jmt = jmt_build(lens);
+        ERR_BAIL(lens->info);
+    }
+
     state->regs = NULL;
     state->nreg = 0;
 
@@ -978,9 +1006,9 @@ static struct frame *rec_process(enum mode_t mode, struct lens *lens,
     visitor.exit = visit_exit;
     visitor.error = visit_error;
     visitor.data = &rec_state;
-    jmt_visit(&visitor, &len);
+    r = jmt_visit(&visitor, &len);
     ERR_BAIL(lens->info);
-    if (len < end - start || (len == 0 && rec_state.fused == 0)) {
+    if (r != 1) {
         get_error(state, lens, "Syntax error");
         state->error->pos = start + len;
     }
@@ -1043,6 +1071,9 @@ static struct tree *get_lens(struct lens *lens, struct state *state) {
     case L_STORE:
         tree = get_store(lens, state);
         break;
+    case L_VALUE:
+        tree = get_value(lens, state);
+        break;
     case L_KEY:
         tree = get_key(lens, state);
         break;
@@ -1070,13 +1101,11 @@ static struct tree *get_lens(struct lens *lens, struct state *state) {
     case L_MAYBE:
         tree = get_quant_maybe(lens, state);
         break;
-    case L_REC:
-        tree = get_rec(lens, state);
-        break;
     default:
-        assert_error(state, "illegal lens tag %d", lens->tag);
+        BUG_ON(true, state->info, "illegal lens tag %d", lens->tag);
         break;
     }
+ error:
     return tree;
 }
 
@@ -1086,7 +1115,7 @@ static struct tree *get_lens(struct lens *lens, struct state *state) {
 static int init_regs(struct state *state, struct lens *lens, uint size) {
     int r;
 
-    if (lens->tag != L_STAR && lens->tag != L_REC) {
+    if (lens->tag != L_STAR && ! lens->recursive) {
         r = match(state, lens, lens->ctype, size, 0);
         if (r == -1)
             get_error(state, lens, "Input string does not match at all");
@@ -1113,11 +1142,14 @@ struct tree *lns_get(struct info *info, struct lens *lens, const char *text,
     struct state state;
     struct tree *tree = NULL;
     uint size = strlen(text);
-    int partial;
+    int partial, r;
 
     MEMZERO(&state, 1);
-    state.info = *info;
-    state.info.ref = UINT_MAX;
+    r = ALLOC(state.info);
+    ERR_NOMEM(r < 0, info);
+
+    *state.info = *info;
+    state.info->ref = UINT_MAX;
 
     state.text = text;
 
@@ -1128,8 +1160,12 @@ struct tree *lns_get(struct info *info, struct lens *lens, const char *text,
      * fails, we throw our arms in the air and say 'something went wrong'
      */
     partial = init_regs(&state, lens, size);
-    if (partial >= 0)
-        tree = get_lens(lens, &state);
+    if (partial >= 0) {
+        if (lens->recursive)
+            tree = get_rec(lens, &state);
+        else
+            tree = get_lens(lens, &state);
+    }
 
     free_seqs(state.seqs);
     if (state.key != NULL) {
@@ -1144,7 +1180,9 @@ struct tree *lns_get(struct info *info, struct lens *lens, const char *text,
         get_error(&state, lens, "Get did not match entire input");
     }
 
+ error:
     free_regs(&state);
+    FREE(state.info);
 
     if (err != NULL) {
         *err = state.error;
@@ -1169,6 +1207,9 @@ static struct skel *parse_lens(struct lens *lens, struct state *state,
     case L_STORE:
         skel = parse_store(lens, state);
         break;
+    case L_VALUE:
+        skel = parse_value(lens, state);
+        break;
     case L_KEY:
         skel = parse_key(lens, state);
         break;
@@ -1196,13 +1237,11 @@ static struct skel *parse_lens(struct lens *lens, struct state *state,
     case L_MAYBE:
         skel = parse_quant_maybe(lens, state, dict);
         break;
-    case L_REC:
-        skel = parse_rec(lens, state, dict);
-        break;
     default:
-        assert_error(state, "illegal lens tag %d", lens->tag);
+        BUG_ON(true, state->info, "illegal lens tag %d", lens->tag);
         break;
     }
+ error:
     return skel;
 }
 
@@ -1211,11 +1250,13 @@ struct skel *lns_parse(struct lens *lens, const char *text, struct dict **dict,
     struct state state;
     struct skel *skel = NULL;
     uint size = strlen(text);
-    int partial;
+    int partial, r;
 
     MEMZERO(&state, 1);
-    state.info.ref = UINT_MAX;
-    state.info.error = lens->info->error;
+    r = ALLOC(state.info);
+    ERR_NOMEM(r< 0, lens->info);
+    state.info->ref = UINT_MAX;
+    state.info->error = lens->info->error;
     state.text = text;
 
     state.text = text;
@@ -1223,7 +1264,10 @@ struct skel *lns_parse(struct lens *lens, const char *text, struct dict **dict,
     partial = init_regs(&state, lens, size);
     if (! partial) {
         *dict = NULL;
-        skel = parse_lens(lens, &state, dict);
+        if (lens->recursive)
+            skel = parse_rec(lens, &state, dict);
+        else
+            skel = parse_lens(lens, &state, dict);
 
         free_seqs(state.seqs);
         if (state.error != NULL) {
@@ -1245,8 +1289,9 @@ struct skel *lns_parse(struct lens *lens, const char *text, struct dict **dict,
         get_error(&state, lens, "parse can not process entire input");
     }
 
+ error:
     free_regs(&state);
-
+    FREE(state.info);
     if (err != NULL) {
         *err = state.error;
     } else {
