@@ -1,7 +1,7 @@
 /*
  * lens.c:
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -54,7 +54,8 @@ static struct value *typecheck_maybe(struct info *info, struct lens *l);
 
 /* Lens names for pretty printing */
 static const char *const tags[] = {
-    "del", "store", "key", "label", "seq", "counter", "concat", "union",
+    "del", "store", "value", "key", "label", "seq", "counter",
+    "concat", "union",
     "subtree", "star", "maybe", "rec"
 };
 
@@ -73,6 +74,21 @@ char *format_lens(struct lens *l) {
     return result;
 }
 
+#define BUG_LENS_TAG(lns)  bug_lens_tag(lns, __FILE__, __LINE__)
+
+static void bug_lens_tag(struct lens *lens, const char *file, int lineno) {
+    char *s = format_lens(lens);
+
+    if (lens != NULL && lens->info != NULL && lens->info->error != NULL) {
+        bug_on(lens->info->error, file, lineno, "Unexpected lens tag %s", s);
+    } else {
+        /* We are really screwed */
+        assert(0);
+    }
+    free(s);
+    return;
+}
+
 /* Construct a finite automaton from REGEXP and return it in *FA.
  *
  * Return NULL if REGEXP is valid, if the regexp REGEXP has syntax errors,
@@ -156,6 +172,7 @@ static struct lens *make_lens_binop(enum lens_tag tag, struct info *info,
     lens->nchildren += (l2->tag == tag) ? l2->nchildren : 1;
 
     lens->recursive = l1->recursive || l2->recursive;
+    lens->rec_internal = l1->rec_internal || l2->rec_internal;
 
     if (ALLOC_N(lens->children, lens->nchildren) < 0) {
         lens->nchildren = 0;
@@ -186,18 +203,21 @@ static struct lens *make_lens_binop(enum lens_tag tag, struct info *info,
     if (ALLOC_N(types, lens->nchildren) < 0)
         goto error;
 
-    if (!lens->recursive) {
+    if (! lens->rec_internal) {
+        /* Inside a recursive lens, we assign types with lns_check_rec
+         * once we know the entire lens */
         for (int t=0; t < ntypes; t++) {
+            if (lens->recursive && t == CTYPE)
+                continue;
             for (int i=0; i < lens->nchildren; i++)
                 types[i] = ltype(lens->children[i], t);
             ltype(lens, t) = (*combinator)(info, lens->nchildren, types);
         }
     }
-
     FREE(types);
 
     for (int i=0; i < lens->nchildren; i++)
-        assert(tag != lens->children[i]->tag);
+        ensure(tag != lens->children[i]->tag, lens->info);
 
     return lens;
  error:
@@ -220,7 +240,7 @@ struct value *lns_make_union(struct info *info,
     int recursive = l1->recursive || l2->recursive;
     int ctype_nullable = l1->ctype_nullable || l2->ctype_nullable;
 
-    if (check && !recursive) {
+    if (check) {
         struct value *exn = typecheck_union(info, l1, l2);
         if (exn != NULL)
             return exn;
@@ -240,11 +260,10 @@ struct value *lns_make_concat(struct info *info,
     int recursive = l1->recursive || l2->recursive;
     int ctype_nullable = l1->ctype_nullable && l2->ctype_nullable;
 
-    if (check && !recursive) {
+    if (check) {
         struct value *exn = typecheck_concat(info, l1, l2);
-        if (exn != NULL) {
+        if (exn != NULL)
             return exn;
-        }
     }
     if (l1->value && l2->value) {
         return make_exn_value(info, "Multiple stores in concat");
@@ -313,6 +332,7 @@ struct value *lns_make_subtree(struct info *info, struct lens *l) {
         lens->atype = subtree_atype(info, l->ktype, l->vtype);
     lens->value = lens->key = 0;
     lens->recursive = l->recursive;
+    lens->rec_internal = l->rec_internal;
     if (! l->recursive)
         lens->ctype_nullable = l->ctype_nullable;
     return make_lens_value(lens);
@@ -321,11 +341,10 @@ struct value *lns_make_subtree(struct info *info, struct lens *l) {
 struct value *lns_make_star(struct info *info, struct lens *l, int check) {
     struct lens *lens;
 
-    if (check && !l->recursive) {
+    if (check) {
         struct value *exn = typecheck_iter(info, l);
-        if (exn != NULL) {
+        if (exn != NULL)
             return exn;
-        }
     }
     if (l->value) {
         return make_exn_value(info, "Multiple stores in iteration");
@@ -339,6 +358,7 @@ struct value *lns_make_star(struct info *info, struct lens *l, int check) {
         ltype(lens, t) = regexp_iter(info, ltype(l, t), 0, -1);
     }
     lens->recursive = l->recursive;
+    lens->rec_internal = l->rec_internal;
     lens->ctype_nullable = 1;
     return make_lens_value(lens);
 }
@@ -358,11 +378,10 @@ struct value *lns_make_plus(struct info *info, struct lens *l, int check) {
 struct value *lns_make_maybe(struct info *info, struct lens *l, int check) {
     struct lens *lens;
 
-    if (check && !l->recursive) {
+    if (check) {
         struct value *exn = typecheck_maybe(info, l);
-        if (exn != NULL) {
+        if (exn != NULL)
             return exn;
-        }
     }
     lens = make_lens_unop(L_MAYBE, info, l);
     for (int t=0; t < ntypes; t++)
@@ -370,6 +389,7 @@ struct value *lns_make_maybe(struct info *info, struct lens *l, int check) {
     lens->value = l->value;
     lens->key = l->key;
     lens->recursive = l->recursive;
+    lens->rec_internal = l->rec_internal;
     lens->ctype_nullable = 1;
     return make_lens_value(lens);
 }
@@ -392,27 +412,30 @@ static struct regexp *make_regexp_from_string(struct info *info,
 
 static struct regexp *restrict_regexp(struct regexp *r) {
     char *nre = NULL;
+    struct regexp *result = NULL;
     size_t nre_len;
     int ret;
 
     ret = fa_restrict_alphabet(r->pattern->str, strlen(r->pattern->str),
                                &nre, &nre_len,
                                RESERVED_FROM, RESERVED_TO);
-    assert(nre_len == strlen(nre));
-    // FIXME: Tell the user what's wrong
-    if (ret != 0)
-        return NULL;
+    ERR_NOMEM(ret == REG_ESPACE || ret < 0, r->info);
+    BUG_ON(ret != 0, r->info, NULL);
+    ensure(nre_len == strlen(nre), r->info);
 
     ret = regexp_c_locale(&nre, &nre_len);
-    if (ret < 0) {
-        free(nre);
-        return NULL;
-    }
+    ERR_NOMEM(ret < 0, r->info);
 
-    r = make_regexp(r->info, nre, r->nocase);
-    if (regexp_compile(r) != 0)
-        abort();
-    return r;
+    result = make_regexp(r->info, nre, r->nocase);
+    nre = NULL;
+    BUG_ON(regexp_compile(result) != 0, r->info,
+           "Could not compile restricted regexp");
+ done:
+    free(nre);
+    return result;
+ error:
+    unref(result, regexp);
+    goto done;
 }
 
 struct value *lns_make_prim(enum lens_tag tag, struct info *info,
@@ -472,18 +495,20 @@ struct value *lns_make_prim(enum lens_tag tag, struct info *info,
     lens->regexp = regexp;
     lens->string = string;
     lens->key = (tag == L_KEY || tag == L_LABEL || tag == L_SEQ);
-    lens->value = (tag == L_STORE);
-    lens->consumes_value = (tag == L_STORE);
+    lens->value = (tag == L_STORE || tag == L_VALUE);
+    lens->consumes_value = (tag == L_STORE || tag == L_VALUE);
     lens->atype = regexp_make_empty(info);
     /* Set the ctype */
     if (tag == L_DEL || tag == L_STORE || tag == L_KEY) {
         lens->ctype = ref(regexp);
         lens->ctype_nullable = regexp_matches_empty(lens->ctype);
-    } else if (tag == L_LABEL || tag == L_SEQ || tag == L_COUNTER) {
+    } else if (tag == L_LABEL || tag == L_VALUE
+               || tag == L_SEQ || tag == L_COUNTER) {
         lens->ctype = regexp_make_empty(info);
         lens->ctype_nullable = 1;
     } else {
-        assert(0);
+        BUG_LENS_TAG(lens);
+        goto error;
     }
 
 
@@ -504,6 +529,8 @@ struct value *lns_make_prim(enum lens_tag tag, struct info *info,
     /* Set the vtype */
     if (tag == L_STORE) {
         lens->vtype = restrict_regexp(lens->regexp);
+    } else if (tag == L_VALUE) {
+        lens->vtype = make_regexp_literal(info, lens->string->str);
     }
 
     return make_lens_value(lens);
@@ -586,10 +613,10 @@ static struct value *typecheck_union(struct info *info,
     return exn;
 }
 
-static struct value *ambig_check(struct info *info,
-                                 struct fa *fa1, struct fa *fa2,
-                                 struct regexp *r1, struct regexp *r2,
-                                 const char *msg, bool iterated) {
+static struct value *
+ambig_check(struct info *info, struct fa *fa1, struct fa *fa2,
+            enum lens_type typ,  struct lens *l1, struct lens *l2,
+            const char *msg, bool iterated) {
     char *upv, *pv, *v;
     size_t upv_len;
     struct value *exn = NULL;
@@ -607,13 +634,26 @@ static struct value *ambig_check(struct info *info,
     }
 
     if (upv != NULL) {
-        char *e_u = escape(upv, pv - upv);
-        char *e_up = escape(upv, v - upv);
-        char *e_upv = escape(upv, -1);
-        char *e_pv = escape(pv, -1);
-        char *e_v = escape(v, -1);
-        char *s1 = regexp_escape(r1);
-        char *s2 = regexp_escape(r2);
+        char *e_u, *e_up, *e_upv, *e_pv, *e_v;
+        char *s1, *s2;
+
+        if (typ == ATYPE) {
+            e_u = enc_format(upv, pv - upv);
+            e_up = enc_format(upv, v - upv);
+            e_upv = enc_format(upv, upv_len);
+            e_pv = enc_format(pv, strlen(pv));
+            e_v = enc_format(v, strlen(v));
+            lns_format_atype(l1, &s1);
+            lns_format_atype(l2, &s2);
+        } else {
+            e_u = escape(upv, pv - upv);
+            e_up = escape(upv, v - upv);
+            e_upv = escape(upv, -1);
+            e_pv = escape(pv, -1);
+            e_v = escape(v, -1);
+            s1 = regexp_escape(ltype(l1, typ));
+            s2 = regexp_escape(ltype(l2, typ));
+        }
         exn = make_exn_value(ref(info), "%s", msg);
         if (iterated) {
             exn_printf_line(exn, "  Iterated regexp: /%s/", s1);
@@ -637,11 +677,14 @@ static struct value *ambig_check(struct info *info,
     return exn;
 }
 
-static struct value *ambig_concat_check(struct info *info, const char *msg,
-                                        struct regexp *r1, struct regexp *r2) {
+static struct value *
+ambig_concat_check(struct info *info, const char *msg,
+                   enum lens_type typ, struct lens *l1, struct lens *l2) {
     struct fa *fa1 = NULL;
     struct fa *fa2 = NULL;
     struct value *result = NULL;
+    struct regexp *r1 = ltype(l1, typ);
+    struct regexp *r2 = ltype(l2, typ);
 
     if (r1 == NULL || r2 == NULL)
         return NULL;
@@ -654,7 +697,7 @@ static struct value *ambig_concat_check(struct info *info, const char *msg,
     if (result != NULL)
         goto done;
 
-    result = ambig_check(info, fa1, fa2, r1, r2, msg, false);
+    result = ambig_check(info, fa1, fa2, typ, l1, l2, msg, false);
  done:
     fa_free(fa1);
     fa_free(fa2);
@@ -666,10 +709,10 @@ static struct value *typecheck_concat(struct info *info,
     struct value *result = NULL;
 
     result = ambig_concat_check(info, "ambiguous concatenation",
-                                l1->ctype, l2->ctype);
+                                CTYPE, l1, l2);
     if (result == NULL) {
         result = ambig_concat_check(info, "ambiguous tree concatenation",
-                                    l1->atype, l2->atype);
+                                    ATYPE, l1, l2);
     }
     if (result != NULL) {
         char *fi = format_info(l1->info);
@@ -682,10 +725,12 @@ static struct value *typecheck_concat(struct info *info,
     return result;
 }
 
-static struct value *ambig_iter_check(struct info *info, const char *msg,
-                                      struct regexp *r) {
+static struct value *
+ambig_iter_check(struct info *info, const char *msg,
+                 enum lens_type typ, struct lens *l) {
     struct fa *fas = NULL, *fa = NULL;
     struct value *result = NULL;
+    struct regexp *r = ltype(l, typ);
 
     if (r == NULL)
         return NULL;
@@ -696,7 +741,7 @@ static struct value *ambig_iter_check(struct info *info, const char *msg,
 
     fas = fa_iter(fa, 0, -1);
 
-    result = ambig_check(info, fa, fas, r, r, msg, true);
+    result = ambig_check(info, fa, fas, typ, l, l, msg, true);
 
  done:
     fa_free(fa);
@@ -707,9 +752,9 @@ static struct value *ambig_iter_check(struct info *info, const char *msg,
 static struct value *typecheck_iter(struct info *info, struct lens *l) {
     struct value *result = NULL;
 
-    result = ambig_iter_check(info, "ambiguous iteration", l->ctype);
+    result = ambig_iter_check(info, "ambiguous iteration", CTYPE, l);
     if (result == NULL) {
-        result = ambig_iter_check(info, "ambiguous tree iteration", l->atype);
+        result = ambig_iter_check(info, "ambiguous tree iteration", ATYPE, l);
     }
     if (result != NULL) {
         char *fi = format_info(l->info);
@@ -723,7 +768,7 @@ static struct value *typecheck_maybe(struct info *info, struct lens *l) {
     /* Check (r)? as (<e>|r) where <e> is the empty language */
     struct value *exn = NULL;
 
-    if (regexp_matches_empty(l->ctype)) {
+    if (l->ctype != NULL && regexp_matches_empty(l->ctype)) {
         exn = make_exn_value(ref(info),
                 "illegal optional expression: /%s/ matches the empty word",
                 l->ctype->pattern->str);
@@ -736,7 +781,7 @@ static struct value *typecheck_maybe(struct info *info, struct lens *l) {
            depending on whether the current node has a non NULL value or not
     */
     if (exn == NULL && ! l->consumes_value) {
-        if (regexp_matches_empty(l->atype)) {
+        if (l->atype != NULL && regexp_matches_empty(l->atype)) {
             exn = make_exn_value(ref(info),
                "optional expression matches the empty tree but does not consume a value");
         }
@@ -747,7 +792,7 @@ static struct value *typecheck_maybe(struct info *info, struct lens *l) {
 void free_lens(struct lens *lens) {
     if (lens == NULL)
         return;
-    assert(lens->ref == 0);
+    ensure(lens->ref == 0, lens->info);
 
     switch (lens->tag) {
     case L_DEL:
@@ -761,6 +806,7 @@ void free_lens(struct lens *lens) {
     case L_LABEL:
     case L_SEQ:
     case L_COUNTER:
+    case L_VALUE:
         unref(lens->string, string);
         break;
     case L_SUBTREE:
@@ -781,7 +827,7 @@ void free_lens(struct lens *lens) {
         }
         break;
     default:
-        assert(0);
+        BUG_LENS_TAG(lens);
         break;
     }
 
@@ -791,6 +837,8 @@ void free_lens(struct lens *lens) {
     unref(lens->info, info);
 
     free(lens);
+ error:
+    return;
 }
 
 void lens_release(struct lens *lens) {
@@ -1019,6 +1067,23 @@ static int lns_format_union_atype(struct lens *l, char **buf) {
     return result;
 }
 
+static int lns_format_rec_atype(struct lens *l, char **buf) {
+    int r;
+
+    if (l->rec_internal) {
+        *buf = strdup("<<rec>>");
+        return (*buf == NULL) ? -1 : 0;
+    }
+
+    char *c = NULL;
+    r = lns_format_atype(l->body, &c);
+    if (r < 0)
+        return -1;
+    r = xasprintf(buf, "<<rec:%s>>", c);
+    free(c);
+    return (r < 0) ? -1 : 0;
+}
+
 int lns_format_atype(struct lens *l, char **buf) {
     *buf = NULL;
 
@@ -1027,6 +1092,7 @@ int lns_format_atype(struct lens *l, char **buf) {
     case L_STORE:
     case L_KEY:
     case L_LABEL:
+    case L_VALUE:
     case L_SEQ:
     case L_COUNTER:
         *buf = strdup("");
@@ -1047,11 +1113,14 @@ int lns_format_atype(struct lens *l, char **buf) {
     case L_UNION:
         return lns_format_union_atype(l, buf);
         break;
+    case L_REC:
+        return lns_format_rec_atype(l, buf);
+        break;
     default:
-        assert(0);
+        BUG_LENS_TAG(l);
         break;
     };
-
+    return -1;
 }
 
 /*
@@ -1060,6 +1129,7 @@ int lns_format_atype(struct lens *l, char **buf) {
 struct value *lns_make_rec(struct info *info) {
     struct lens *l = make_lens(L_REC, info);
     l->recursive = 1;
+    l->rec_internal = 1;
 
     return make_lens_value(l);
 }
@@ -1379,7 +1449,8 @@ static void rtn_rules(struct rtn *rtn, struct lens *l) {
             break;
         }
         default:
-            assert(0);
+            BUG_ON(true, rtn->info, "Unexpected lens type %d", rtn->lens_type);
+            break;
         }
         break;
     case L_MAYBE:
@@ -1399,7 +1470,8 @@ static void rtn_rules(struct rtn *rtn, struct lens *l) {
         RTN_BAIL(rtn);
         break;
     default:
-        assert(0);
+        BUG_LENS_TAG(l);
+        break;
     }
  error:
     return;
@@ -1648,7 +1720,7 @@ static struct regexp *rtn_reduce(struct rtn *rtn, struct lens *rec) {
         struct regexp *loop = NULL;
         for (int i=0; i < s->ntrans; i++) {
             if (s == s->trans[i].to) {
-                assert(loop == NULL);
+                ensure(loop == NULL, rtn->info);
                 loop = s->trans[i].re;
             }
         }
@@ -1675,7 +1747,7 @@ static struct regexp *rtn_reduce(struct rtn *rtn, struct lens *rec) {
     struct regexp *result = NULL;
     for (int i=0; i < prod->start->ntrans; i++) {
         if (prod->start->trans[i].to == prod->end) {
-            assert(result == NULL);
+            ensure(result == NULL, rtn->info);
             result = ref(prod->start->trans[i].re);
         }
     }
@@ -1731,7 +1803,8 @@ static void propagate_type(struct lens *l, enum lens_type lt) {
         /* Nothing to do */
         break;
     default:
-        assert(0);
+        BUG_LENS_TAG(l);
+        break;
     }
 
  error:
@@ -1766,7 +1839,7 @@ static struct value *typecheck_n(struct lens *l,
     struct value *exn = NULL;
     struct lens *acc = NULL;
 
-    assert(l->tag == L_CONCAT || l->tag == L_UNION);
+    ensure(l->tag == L_CONCAT || l->tag == L_UNION, l->info);
     for (int i=0; i < l->nchildren; i++) {
         exn = typecheck(l->children[i], check);
         if (exn != NULL)
@@ -1779,7 +1852,7 @@ static struct value *typecheck_n(struct lens *l,
         exn = (*make)(info, acc, ref(l->children[i]), check);
         if (EXN(exn))
             goto error;
-        assert(exn->tag == V_LENS);
+        ensure(exn->tag == V_LENS, l->info);
         acc = ref(exn->lens);
         unref(exn, value);
     }
@@ -1825,7 +1898,8 @@ static struct value *typecheck(struct lens *l, int check) {
         /* Nothing to do */
         break;
     default:
-        assert(0);
+        BUG_LENS_TAG(l);
+        break;
     }
 
     return exn;
@@ -1929,7 +2003,8 @@ static int ctype_nullable(struct lens *lens, struct value **exn) {
         nullable = lens->body->ctype_nullable;
         break;
     default:
-        assert(0);
+        BUG_LENS_TAG(lens);
+        break;
     }
     if (*exn != NULL)
         return 0;
@@ -1945,11 +2020,12 @@ struct value *lns_check_rec(struct info *info,
                             int check) {
     /* The types in the order of approximation */
     static const enum lens_type types[] = { KTYPE, VTYPE, ATYPE };
-
-    assert(rec->tag == L_REC);
-    assert(body->recursive);
     struct value *result = NULL;
 
+    ensure(rec->tag == L_REC, info);
+    ensure(body->recursive, info);
+    ensure(rec->rec_internal, info);
+
     /* To help memory management, we avoid the cycle inherent ina recursive
      * lens by using two instances of an L_REC lens. One is marked with
      * rec_internal, and used inside the body of the lens. The other is the
@@ -1958,7 +2034,6 @@ struct value *lns_check_rec(struct info *info,
      * The internal instance of the recursive lens is REC, the external one
      * is TOP, constructed below
      */
-    rec->rec_internal = 1;
     rec->body = body;                          /* REC does not own BODY */
 
     for (int i=0; i < ARRAY_CARDINALITY(types); i++) {
@@ -1967,7 +2042,7 @@ struct value *lns_check_rec(struct info *info,
     }
 
     if (rec->atype == NULL) {
-        result = make_exn_value(rec->info,
+        result = make_exn_value(ref(rec->info),
         "recursive lens generates the empty language for its %s",
          rec->ctype == NULL ? "ctype" : "atype");
         goto error;
@@ -1986,9 +2061,6 @@ struct value *lns_check_rec(struct info *info,
     if (result != NULL)
         goto error;
 
-    struct jmt *jmt = jmt_build(rec);
-    ERR_BAIL(info);
-
     result = lns_make_rec(ref(rec->info));
     struct lens *top = result->lens;
     for (int t=0; t < ntypes; t++)
@@ -1999,12 +2071,16 @@ struct value *lns_check_rec(struct info *info,
     top->ctype_nullable = rec->ctype_nullable;
     top->body = ref(body);
     top->alias = rec;
+    top->rec_internal = 0;
     rec->alias = top;
 
-    top->jmt = jmt;
+    top->jmt = jmt_build(top);
+    ERR_BAIL(info);
 
     return result;
  error:
+    if (result != NULL && result->tag != V_EXN)
+        unref(result, value);
     if (result == NULL)
         result = exn_error();
     return result;
