@@ -1,7 +1,7 @@
 /*
  * put.c:
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -26,6 +26,7 @@
 #include "regexp.h"
 #include "memory.h"
 #include "lens.h"
+#include "errcode.h"
 
 /* Data structure to keep track of where we are in the tree. The split
  * describes a sublist of the list of siblings in the current tree. The
@@ -81,7 +82,7 @@ static void put_error(struct state *state, struct lens *lens,
     state->error->lens = ref(lens);
     state->error->pos  = -1;
     if (strlen(state->path) == 0) {
-        state->error->path = strdup("(root)");
+        state->error->path = strdup("");
     } else {
         state->error->path = strdup(state->path);
     }
@@ -370,6 +371,8 @@ static int skel_instance_of(struct lens *lens, struct skel *skel) {
         return skel->tag == L_KEY;
     case L_LABEL:
         return skel->tag == L_LABEL;
+    case L_VALUE:
+        return skel->tag == L_VALUE;
     case L_SEQ:
         return skel->tag == L_SEQ;
     case L_COUNTER:
@@ -412,10 +415,10 @@ static int skel_instance_of(struct lens *lens, struct skel *skel) {
     case L_REC:
         return skel_instance_of(lens->body, skel);
     default:
-        assert_error_at(__FILE__, __LINE__, lens->info,
-                        "illegal lens tag %d", lens->tag);
+        BUG_ON(true, lens->info, "illegal lens tag %d", lens->tag);
         break;
     }
+ error:
     return 0;
 }
 
@@ -502,28 +505,50 @@ static void put_concat(struct lens *lens, struct state *state) {
     state->skel = oldskel;
 }
 
+static void error_quant_star(struct split *last_split, struct lens *lens,
+                             struct state *state) {
+    struct tree *child = NULL;
+    if (last_split != NULL) {
+        if (last_split->follow != NULL) {
+            child = last_split->follow;
+        } else {
+            for (child = last_split->tree;
+                 child != NULL && child->next != NULL;
+                 child = child->next);
+        }
+    }
+    if (child == NULL) {
+        put_error(state, lens, "Malformed child node");
+    } else {
+        put_error(state, lens, "Malformed child node '%s'", child->label);
+    }
+}
+
 static void put_quant_star(struct lens *lens, struct state *state) {
     assert(lens->tag == L_STAR);
     struct split *oldsplit = state->split;
     struct skel *oldskel = state->skel;
+    struct split *last_split = NULL;
 
     struct split *split = split_iter(state, lens);
 
     state->skel = state->skel->skels;
     set_split(state, split);
+    last_split = state->split;
     while (state->split != NULL && state->skel != NULL) {
         put_lens(lens->child, state);
         state->skel = state->skel->next;
+        last_split = state->split;
         next_split(state);
     }
     while (state->split != NULL) {
         create_lens(lens->child, state);
+        last_split = state->split;
         next_split(state);
     }
+    if (state->pos != oldsplit->end)
+        error_quant_star(last_split, lens, state);
     list_free(split);
-    if (state->pos != oldsplit->end) {
-        put_error(state, lens, "Short iteration");
-    }
     set_split(state, oldsplit);
     state->skel = oldskel;
 }
@@ -575,6 +600,7 @@ static void put_lens(struct lens *lens, struct state *state) {
         fprintf(state->out, "%s", state->key);
         break;
     case L_LABEL:
+    case L_VALUE:
         /* Nothing to do */
         break;
     case L_SEQ:
@@ -653,18 +679,20 @@ static void create_concat(struct lens *lens, struct state *state) {
 static void create_quant_star(struct lens *lens, struct state *state) {
     assert(lens->tag == L_STAR);
     struct split *oldsplit = state->split;
+    struct split *last_split = NULL;
 
     struct split *split = split_iter(state, lens);
 
     set_split(state, split);
+    last_split = state->split;
     while (state->split != NULL) {
         create_lens(lens->child, state);
+        last_split = state->split;
         next_split(state);
     }
+    if (state->pos != oldsplit->end)
+        error_quant_star(last_split, lens, state);
     list_free(split);
-    if (state->pos != oldsplit->end) {
-        put_error(state, lens, "Short iteration");
-    }
     set_split(state, oldsplit);
 }
 
@@ -694,6 +722,7 @@ static void create_lens(struct lens *lens, struct state *state) {
         fprintf(state->out, "%s", state->key);
         break;
     case L_LABEL:
+    case L_VALUE:
         /* Nothing to do */
         break;
     case L_SEQ:
