@@ -1,7 +1,7 @@
 /*
  * syntax.c:
  *
- * Copyright (C) 2007, 2008 Red Hat Inc.
+ * Copyright (C) 2007-2010 David Lutterkort
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -52,6 +52,7 @@ static const struct type tree_type      = { .ref = UINT_MAX, .tag = T_TREE };
 static const struct type filter_type    = { .ref = UINT_MAX, .tag = T_FILTER };
 static const struct type transform_type =
                                        { .ref = UINT_MAX, .tag = T_TRANSFORM };
+static const struct type unit_type      = { .ref = UINT_MAX, .tag = T_UNIT };
 
 const struct type *const t_string    = &string_type;
 const struct type *const t_regexp    = &regexp_type;
@@ -59,12 +60,16 @@ const struct type *const t_lens      = &lens_type;
 const struct type *const t_tree      = &tree_type;
 const struct type *const t_filter    = &filter_type;
 const struct type *const t_transform = &transform_type;
+const struct type *const t_unit      = &unit_type;
 
 static const char *const type_names[] = {
     "string", "regexp", "lens", "tree", "filter",
-    "transform", "function", NULL
+    "transform", "unit", "function", NULL
 };
 
+/* The anonymous identifier which we will never bind */
+static const char const anon_ident[] = "_";
+
 static void print_value(FILE *out, struct value *v);
 
 /* The evaluation context with all loaded modules and the bindings for the
@@ -132,19 +137,6 @@ void fatal_error(struct info *info, const char *format, ...) {
     va_end(ap);
 }
 
-void assert_error_at(const char *srcfile, int srclineno, struct info *info,
-                     const char *format, ...) {
-    va_list ap;
-
-    fprintf(stderr, "%s:%d:(", srcfile, srclineno);
-    print_info(stderr, info);
-    fprintf(stderr,"):Internal error:");
-	va_start(ap, format);
-    vfprintf(stderr, format, ap);
-    va_end(ap);
-    fprintf(stderr, "\n");
-}
-
 static void free_param(struct param *param) {
     if (param == NULL)
         return;
@@ -291,6 +283,8 @@ void free_value(struct value *v) {
     case V_EXN:
         free_exn(v->exn);
         break;
+    case V_UNIT:
+        break;
     default:
         assert(0);
     }
@@ -341,6 +335,10 @@ struct value *make_value(enum value_tag tag, struct info *info) {
     return value;
 }
 
+struct value *make_unit(struct info *info) {
+    return make_value(V_UNIT, info);
+}
+
 struct term *make_app_term(struct term *lambda, struct term *arg,
                            struct info *info) {
   struct term *app = make_term(A_APP, info);
@@ -572,6 +570,9 @@ static struct type *ctx_lookup_type(struct info *info,
 static struct binding *bind_type(struct binding **bnds,
                                  const char *name, struct type *type) {
     struct binding *binding;
+
+    if (STREQ(name, anon_ident))
+        return NULL;
     make_ref(binding);
     make_ref(binding->ident);
     binding->ident->str = strdup(name);
@@ -601,11 +602,15 @@ static void unbind_param(struct binding **bnds, ATTRIBUTE_UNUSED struct param *p
     unref(b, binding);
 }
 
-/* Takes ownership of TYPE and VALUE */
+/* Takes ownership of VALUE */
 static void bind(struct binding **bnds,
                  const char *name, struct type *type, struct value *value) {
-    struct binding *b = bind_type(bnds, name, type);
-    b->value = ref(value);
+    struct binding *b = NULL;
+
+    if (STRNEQ(name, anon_ident)) {
+        b = bind_type(bnds, name, type);
+        b->value = ref(value);
+    }
 }
 
 /*
@@ -718,6 +723,9 @@ static void print_value(FILE *out, struct value *v) {
             v->exn->seen = 1;
         }
         break;
+    case V_UNIT:
+        fprintf(out, "()");
+        break;
     default:
         assert(0);
         break;
@@ -789,6 +797,8 @@ struct type *make_base_type(enum type_tag tag) {
         return (struct type *) t_filter;
     else if (tag == T_TRANSFORM)
         return (struct type *) t_transform;
+    else if (tag == T_UNIT)
+        return (struct type *) t_unit;
     assert(0);
     abort();
 }
@@ -832,15 +842,11 @@ static int subtype(struct type *t1, struct type *t2) {
        of strings/regexps to lenses (yet) */
     if (t1->tag == T_STRING)
         return (t2->tag == T_STRING || t2->tag == T_REGEXP);
-    if (t1->tag == T_REGEXP)
-        return t2->tag == T_REGEXP;
-    if (t1->tag == T_LENS)
-        return t2->tag == T_LENS;
     if (t1->tag == T_ARROW && t2->tag == T_ARROW) {
         return subtype(t2->dom, t1->dom)
             && subtype(t1->img, t2->img);
     }
-    return 0;
+    return t1->tag == t2->tag;
 }
 
 static int type_equal(struct type *t1, struct type *t2) {
@@ -916,6 +922,8 @@ static struct type *value_type(struct value *v) {
         return make_base_type(T_FILTER);
     case V_TRANSFORM:
         return make_base_type(T_TRANSFORM);
+    case V_UNIT:
+        return make_base_type(T_UNIT);
     case V_NATIVE:
         return ref(v->native->type);
     case V_CLOS:
@@ -1108,6 +1116,10 @@ static int check_compose(struct term *term, struct ctx *ctx) {
         if (! subtype(tl->img, tr->dom))
             goto print_error;
         term->type = make_arrow_type(tl->dom, tr->img);
+    } else if (tl->tag == T_UNIT) {
+        if (! check_exp(term->right, ctx))
+            return 0;
+        term->type = ref(term->right->type);
     } else {
         goto print_error;
     }
@@ -1464,9 +1476,9 @@ static struct value *compile_compose(struct term *exp, struct ctx *ctx) {
         v = make_closure(func, ctx->local);
         unref(func, term);
     } else {
-        fatal_error(info, "Tried to compose a %s and a %s to yield a %s",
-                    type_name(exp->left->type), type_name(exp->right->type),
-                    type_name(t));
+        v = compile_exp(exp->info, exp->left, ctx);
+        unref(v, value);
+        v = compile_exp(exp->info, exp->right, ctx);
     }
     return v;
 }
