@@ -43,7 +43,8 @@ static void print_regerror(int err, const char *regexp) {
     size_t size;
     char *errbuf;
     size = regerror(err, NULL, NULL, 0);
-    errbuf = alloca(size);
+    if (ALLOC_N(errbuf, size) < 0)
+        die_oom();
     regerror(err, NULL, errbuf, size);
     if (strlen(regexp) > 40) {
         char *s = strndup(regexp, 40);
@@ -53,6 +54,7 @@ static void print_regerror(int err, const char *regexp) {
         fprintf(stderr, "Error building fa from %s:\n", regexp);
     }
     fprintf(stderr, "  %s\n", errbuf);
+    free(errbuf);
 }
 
 static void setup(ATTRIBUTE_UNUSED CuTest *tc) {
@@ -355,12 +357,18 @@ static void testExample(CuTest *tc) {
     assertExample(tc, "vu{2}|[0-9]", "0");
     assertExample(tc, "\\[", "[");
     assertExample(tc, "[\\]", "\\");
+    assertExample(tc, "a{3}", "aaa");
+    assertExample(tc, "a{3,}", "aaa");
 
     assertExample(tc, "\001((\002.)*\001)+\002", "\001\001\002");
     assertExample(tc, "\001((\001.)*\002)+\002", "\001\002\002");
 
     assertExample(tc, "a[^\001-\004]+b", "aAb");
 
+    /* A strange way to write a? - allowed by POSIX */
+    assertExample(tc, "(a|)", "a");
+    assertExample(tc, "(|a)", "a");
+
     struct fa *fa1 = mark(fa_make_basic(FA_EMPTY));
     size_t xmpl_len;
     char *xmpl;
@@ -511,6 +519,8 @@ static void testRestrictAlphabet(CuTest *tc) {
 
 static void testExpandCharRanges(CuTest *tc) {
     const char *re = "[1-3]*|[a-b]([^\nU-X][^\n])*";
+    const char *re2 = "a\\|b";
+
     char *nre;
     size_t nre_len;
     int r;
@@ -519,6 +529,12 @@ static void testExpandCharRanges(CuTest *tc) {
     CuAssertIntEquals(tc, 0, r);
     CuAssertStrEquals(tc, "[123]*|[ab]([^\nUVWX].)*", nre);
     CuAssertIntEquals(tc, strlen(nre), nre_len);
+    free(nre);
+
+    r = fa_expand_char_ranges(re2, strlen(re2), &nre, &nre_len);
+    CuAssertIntEquals(tc, 0, r);
+    CuAssertStrEquals(tc, re2, nre);
+    free(nre);
 }
 
 static void testNoCase(CuTest *tc) {
