@@ -1,6 +1,6 @@
 # -*- ruby -*-
 commands="
-rm /files/etc/pam.d/newrole/4/opts
+rm /files/etc/pam.d/newrole/4/argument
 save
 "
 refresh = 1
